"""pynomer - A python wrapper for nomer"""

from .pynomer import *

__version__ = "0.1.2"
__author__ = "nleguillarme <nicolas.leguillarme@univ-grenoble-alpes.fr>"
__all__ = [
    "version",
    "clean",
    "input_schema",
    "output_schema",
    "properties",
    "matchers",
    "validate_term",
    "validate_term_link",
    "replace",
    "append",
]



import urllib
import requests
import json
import pynomer
import click
import logging

logger = logging.getLogger(__name__)
logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.INFO)


class NoResultException(Exception):
    pass


@click.group()
def cli():
    pass


@cli.command()
def version():
    """
        Show Version.

        Usage::

            pynomer version
        """
    click.echo(pynomer.version())


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
def clean(properties):
    """
        Cleans term matcher cache.

        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer clean
        """
    click.echo(pynomer.clean(properties))


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
def input_schema(properties):
    """
        Show input schema in JSON.

        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer input_schema
            pynomer input_schema -p ./new_properties.txt
        """
    click.echo(pynomer.input_schema(properties))


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
def output_schema(properties):
    """
        Show output schema.

        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer output_schema
            pynomer output_schema -p ./new_properties.txt
        """
    click.echo(pynomer.output_schema(properties))


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
def properties(properties):
    """
        Lists configuration properties. Can be used to make a local copy and override
        default settings.

        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer properties
            pynomer properties -p ./new_properties.txt
        """
    click.echo(pynomer.properties(properties))


@cli.command()
@click.option(
    "--output-format",
    "-o",
    type=str,
    help="""
            tsv, json""",
    default="tsv",
)
@click.option(
    "--verbose",
    "-v",
    type=bool,
    help="""
            if set, matcher descriptions are included for tsv.""",
    default=False,
)
def matchers(output_format, verbose):
    """
        Lists supported matcher and (optionally) their descriptions.

        :param o: ["tsv", "json"]  Output format. Default: "tsv"
        :param v: [bool]  If set, matcher descriptions are included for tsv. Default: False

        Usage::

            pynomer matchers
        """
    click.echo(pynomer.matchers(output_format, verbose))


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
@click.argument("filepath", required=True)
def validate_term(filepath, properties):
    """
        Validate terms.

        :param filepath: [string]  Path or URL to TaxonCache file. Default: <empty string>
        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer validate_term https://zenodo.org/record/1213465/files/taxonCacheFirst10.tsv
        """
    click.echo(pynomer.validate_term(filepath, properties))


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
@click.argument("filepath", required=True)
def validate_term_link(filepath, properties):
    """
        Validate term links.

        :param filepath: [string]  Path or URL to TaxonMap file. Default: <empty string>
        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer validate_term_link
        """
    click.echo(pynomer.validate_term_link(filepath, properties))


@cli.command()
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
@click.option(
    "--matcher",
    "-m",
    type=str,
    help="""
            Default: globi-taxon-cache""",
    default="globi-taxon-cache",
)
@click.option(
    "-e",
    is_flag=True,
    help="""
            Activate echo -e option""",
)
@click.argument("query", required=True, default="")
def replace(query, matcher, properties, e):
    """
        Replace exact term matches in row. The input schema is used
        to select the id and/or name to match to. The output schema is
        used to select the columns to write into.

        :param query: [string]  Query. Default: <empty string>
        :param matcher: [string]  Selected matcher. Default: "globi-taxon-cache"
        :param p: [string]  Path to properties file to override defaults. Default: None

        Usage::

            pynomer replace "\\tHomo sapiens"

            pynomer replace "ITIS:180547\\t" -m globi-enrich
        """
    click.echo(pynomer.replace(query, matcher, properties, echo_opt="-e" if e else ""))


@cli.command()
@click.option(
    "--output-format",
    "-o",
    type=str,
    help="""
            tsv, json""",
    default="tsv",
)
@click.option(
    "--properties",
    "-p",
    type=str,
    help="""
            Path to properties file to override defaults.
            Default: <empty string>""",
    default="",
)
@click.option(
    "--matcher",
    "-m",
    type=str,
    help="""
            Default: globi-taxon-cache""",
    default="globi-taxon-cache",
)
@click.option(
    "-e",
    is_flag=True,
    help="""
            Activate echo -e option""",
)
@click.argument("query", required=True, default="")
def append(query, matcher, properties, output_format, e):
    """
        Append term match to row using id and name columns specified
        in input schema. Multiple matches result in multiple rows.

        :param query: [string]  Query. Default: <empty string>
        :param matcher: [string]  Selected matcher. Default: "globi-taxon-cache"
        :param p: [string]  Path to properties file to override defaults. Default: None
        :param o: ["tsv", "json"]  Output format. Default: "tsv"

        Usage::

            pynomer append "\\tHomo sapiens"

            pynomer append "ITIS:180547\\t" -m globi-enrich
        """
    click.echo(
        pynomer.append(
            query, matcher, properties, output_format, echo_opt="-e" if e else ""
        )
    )


def main():
    cli()



import logging
import subprocess
import json
import sys

logger = logging.getLogger(__name__)


class NomerException(Exception):
    pass


# validate-term, validate-term-link
def get_nomer_validate_cmd(filepath="", cmd="validate-term", properties=None):
    if filepath == "":
        raise ValueError("Filepath cannot be empty strings")
    nomer_cmd = "curl -L {} | nomer {}".format(filepath, cmd)
    if properties:
        nomer_cmd = "{} -p {}".format(nomer_cmd, properties)
    return nomer_cmd


# replace, append
def get_nomer_match_cmd(
    query="",
    cmd="append",
    matcher="globi-taxon-cache",
    properties=None,
    output_format=None,
    echo_opt="",
):
    # if id == "" and name == "":
    #     raise ValueError("Id and name cannot be empty strings")
    # query = r"'{}'".format(query)
    nomer_cmd = "echo {} '{}' | nomer {} {} -Xmx4096m -Xms1024m".format(
        echo_opt, query, cmd, matcher
    )
    if properties:
        nomer_cmd = "{} -p {}".format(nomer_cmd, properties)
    if output_format:
        nomer_cmd = "{} -o {}".format(nomer_cmd, output_format)
    return nomer_cmd


def get_nomer_simple_cmd(
    cmd="version", verbose=False, properties=None, output_format=None
):
    nomer_cmd = "nomer {}".format(cmd)
    if properties:
        nomer_cmd = "{} -p {}".format(nomer_cmd, properties)
    if output_format:
        nomer_cmd = "{} -o {}".format(nomer_cmd, output_format)
    if verbose:
        nomer_cmd = "{} -v".format(nomer_cmd)
    return nomer_cmd


def run_nomer(nomer_cmd):
    logger.debug("Run nomer command {}".format(nomer_cmd))
    p = subprocess.Popen(
        nomer_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL
    )
    result = p.communicate()[0].decode("utf8")
    rc = p.returncode
    if rc:
        raise NomerException(
            "Command {} got return code {}. This is probably due to nomer throwing an exception.".format(
                nomer_cmd, rc
            )
        )
    logger.debug(
        "Command {} got return code {} and result {}".format(nomer_cmd, rc, result)
    )
    if result:
        return nomer_cmd, result.strip("\n")
    return nomer_cmd, None



"""
PyNomer is a simple python wrapper for nomer
"""
from setuptools import find_packages, setup

dependencies = ["click", "setuptools", "requests"]

# read the contents of your README file
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="pynomer",
    version="v0.1.3",
    url="https://github.com/nleguillarme/pynomer",
    license="MIT",
    author="Nicolas Le Guillarme",
    author_email="nicolas.leguillarme@univ-grenoble-alpes.fr",
    description="A python wrapper for nomer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    zip_safe=False,
    platforms="any",
    install_requires=dependencies,
    entry_points={"console_scripts": ["pynomer = pynomer.cli:main"]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
    ],
)
